"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const readPkgUp = require("read-pkg-up");
const resolvePkg = require("resolve-pkg");
const os_1 = require("os");
const path_1 = require("path");
const fs_1 = require("fs");
const child_process_1 = require("child_process");
const errors_1 = require("./errors");
class ConfigResolver {
    getConfig(projectRoot) {
        let jestScriptPath = null;
        let config = {};
        const configFromPkgJson = this.getMajesticConfigFromPackageJson(projectRoot) || {};
        if (this.isBootstrappedWithCreateReactApp(projectRoot)) {
            jestScriptPath = this.getJestScriptForCreateReactApp(projectRoot);
            config = {
                args: ["--env=jsdom"],
                env: {
                    CI: "true"
                }
            };
        }
        else {
            jestScriptPath =
                configFromPkgJson.jestScriptPath || this.getJestScriptPath(projectRoot);
        }
        const configFromJest = this.getConfigFromJest(jestScriptPath, projectRoot);
        const firstConfig = configFromJest.config ||
            (configFromJest.configs && configFromJest.configs[0]);
        if (!firstConfig) {
            throw new errors_1.UnableToResolveConfig("");
        }
        return Object.assign({}, config, { jestScriptPath, testMatch: firstConfig.testMatch, testRegex: firstConfig.testRegex }, configFromPkgJson);
    }
    getJestScriptPath(projectRoot) {
        const path = resolvePkg("jest", {
            cwd: projectRoot
        });
        if (!path) {
            throw new errors_1.CouldNotResolveJestPath("Unable to find Jest script. But you can provide the path to Jest script via package.json. Have a look at the documentation: https://github.com/Raathigesh/majestic");
        }
        return path_1.join(path, "bin/jest.js");
    }
    getJestScriptForCreateReactApp(projectRoot) {
        const path = resolvePkg("react-scripts", {
            cwd: projectRoot
        });
        return path_1.join(path, "scripts/test.js");
    }
    getPackageJson(rootPath) {
        return readPkgUp.sync({
            cwd: rootPath
        }).pkg;
    }
    getMajesticConfigFromPackageJson(projectRoot) {
        const packageJson = this.getPackageJson(projectRoot);
        if (packageJson.majestic) {
            return packageJson.majestic;
        }
        return null;
    }
    getConfigFromJest(jestScriptPath, projectRoot) {
        const process = child_process_1.spawnSync(`node ${jestScriptPath}`, ["--showConfig"], {
            cwd: projectRoot,
            shell: true,
            stdio: "pipe",
            env: {}
        });
        const configStr = process.stdout.toString().trim();
        const config = JSON.parse(configStr);
        return config;
    }
    isBootstrappedWithCreateReactApp(rootPath) {
        return (this.hasExecutable(rootPath, "node_modules/.bin/react-scripts") ||
            this.hasExecutable(rootPath, "node_modules/react-scripts/node_modules/.bin/jest") ||
            this.hasExecutable(rootPath, "node_modules/react-native-scripts"));
    }
    hasExecutable(rootPath, executablePath) {
        const ext = os_1.platform() === "win32" ? ".cmd" : "";
        const absolutePath = path_1.join(rootPath, executablePath + ext);
        return fs_1.existsSync(absolutePath);
    }
}
exports.default = ConfigResolver;
//# sourceMappingURL=config-resolver.js.map